import CloudFlare
import json
import requests
import schedule
import time


def main():
    config = get_config()
    schedule.every(config.get("interval_minutes")).minutes.do(do_full_cycle)

    do_full_cycle()
    while True:
        schedule.run_pending()
        time.sleep(1)


def do_full_cycle():
    try:
        config = get_config()
        zones: dict = config.get("zones")
        api_key = config.get("api_key")
        public_ip = get_current_public_ip()

        for zone_name in zones.keys():
            sub_domains = zones.get(zone_name)
            update_dns_records_in_zone(zone_name, sub_domains, public_ip, api_key)
    except Exception as e:
        print(e)


def update_dns_records_in_zone(
    zone_name: str, sub_domains: list[str], public_ip: str, api_key: str
):
    cf = CloudFlare.CloudFlare(key=api_key)

    zones = cf.zones.get(params={"name": zone_name, "per_page": 1})

    zone = zones[0]
    zone_id = zone["id"]

    dns_records = cf.zones.dns_records.get(zone_id)

    remaining_sub_domains = sub_domains.copy()
    for dns_record in dns_records:
        r_name = dns_record["name"]
        r_value = dns_record["content"]
        r_id = dns_record["id"]

        matching_sub_domain = [
            sub_domain
            for sub_domain in remaining_sub_domains
            if sub_domain + "." + zone_name == r_name
        ]
        if matching_sub_domain:
            matching_sub_domain = matching_sub_domain[0]
            remaining_sub_domains.remove(matching_sub_domain)

            if r_value != public_ip:
                name = matching_sub_domain + "." + zone_name
                print("{} was outdated. Updating...".format(name))

                cf.zones.dns_records.patch(zone_id, r_id, data={"content": public_ip})

    for sub_domain in remaining_sub_domains:
        name = sub_domain + "." + zone_name
        print("{} does not exist. Creating...".format(name))

        data = {
            "name": name,
            "type": "A",
            "content": public_ip,
            "ttl": 1,
            "proxied": False,
        }
        cf.zones.dns_records.post(zone_id, data=data)


def get_config() -> dict:
    with open("config.json") as config:
        return json.load(config)


def get_current_public_ip() -> str:
    return requests.get("https://api.ipify.org/").text


if __name__ == "__main__":
    main()
